local dt = require "darktable"

local sleep = dt.control.sleep

local image_list = nil
local current_image_count = nil

local SLEEP_TIME = 1500

local complete_flag = 1
local loaded_flag = 1

local mods = {}

mods.colorbalancergb = "iop/colorbalancergb"
mods.color_calibration = "iop/channelmixerrgb"
mods.diffuse_or_sharpen = "iop/diffuse"
mods.denoise_profiled = "iop/denoiseprofile"
mods.exposure = "iop/exposure"
mods.filmicrgb = "iop/filmicrgb"
mods.highlight_reconstruction = "iop/highlights"

local function wait_until_complete()
  while not complete_flag do
    dt.print_log("waiting for complete_flag")
    sleep(250)
  end
  dt.print_log("complete")
end

local function wait_until_loaded()
  while not loaded_flag do
    dt.print_log("waiting for loaded_flag")
    sleep(250)
  end
  dt.print_log("loaded")
end

local function rel_adjust(mod, item, instance, amount)
  local inst = instance or 0
  dt.print_log("in rel_adjust")
  if complete_flag then
    complete_flag = nil
  else
    dt.print_log("error: complete_flag is still nil")
    return 
  end
  local status = dt.gui.action(mod .. "/" .. item, inst, "value", "set", "")
  dt.print_log("status is " .. status .. " and adjusted value should be " .. amount + status)
  dt.gui.action(mod .. "/" .. item, inst, "value", "set", status + amount)
  wait_until_complete()
end

local function process_image(image)
  dt.print("Processing " .. image.filename)
  rel_adjust(mods.exposure, "exposure", 0, 0.5)
end

local function display_images(images)
  dt.print_log("in display images")
  loaded_flag = nil
  dt.gui.current_view(dt.gui.views.darkroom)
  dt.print_log("switched to darkroom view")
  sleep(500)
  dt.gui.views.darkroom.display_image(images[1])
  wait_until_loaded()
  --sleep(500)
  for i = 1, #images do
    dt.print_log("calling process_image")
    process_image(images[i])
    dt.print_log("loading next image\n\n")
    --sleep(1000)
    if i < #images then
      loaded_flag = nil
      dt.print_log("changing image to " .. images[i + 1].filename)
      dt.gui.views.darkroom.display_image(images[i + 1])
      wait_until_loaded()
      --sleep(500)
    end
  end
  dt.print_log("finished processing, returning to lighttable")
  dt.gui.current_view(dt.gui.views.lighttable)
end

dt.gui.libs.image.register_action(
  "display_in_darkroom", "display in darkroom",
  function(event, images) display_images(images) end,
  "Display selected images in darkroom mode"
)

dt.register_event("display_in_darkroom", "darkroom-image-loaded",
  function(event, clean, image)
    dt.print_log("caught darkroom-image-loaded event with clean = " .. tostring(clean))
    if not clean then
      dt.print_log("didn't get a clean load, reloading")
      sleep(100)
      dt.gui.views.darkroom.display_image(image)
    else
      loaded_flag = 1
    end
  end 
)

dt.register_event("display_in_darkroom", "pixelpipe-processing-complete",
  function(event, image)
    dt.print_log("caught pixelpipe-processing-complete")
    complete_flag = 1
  end 
)

